@@ -82,16 +82,28 @@ main_client_manage_toolbars_for_fullscreen(Client *c, Bool main_client_showing)
   Wm     *w = c->wm;
   Client *p  = NULL;
 
+  dbg("%s() called\n", __func__);
+
   stack_enumerate(w, p)
     {
-      if (p->type == MBCLIENT_TYPE_DIALOG 
-	  && p->flags & CLIENT_TB_ALT_TRANS_FOR_APP
-	  && p->trans == c)
+      if (p->type == MBCLIENT_TYPE_DIALOG)
 	{
-	  toolbar_client_configure(p);
-	  toolbar_client_move_resize(p);
+	  if (p->flags & CLIENT_TB_ALT_TRANS_FOR_APP 
+	      && p->trans == c)
+	    {
+	      toolbar_client_configure(p);
+	      toolbar_client_move_resize(p);
 
-	  return p->height;
+	      return p->height;
+	    }
+	  else if (p->flags & CLIENT_TB_ALT_TRANS_FOR_DIALOG)
+	    {
+	      dbg("%s() IM trans for dialog found resizing..\n", __func__);
+
+	      toolbar_client_configure(p);
+	      toolbar_client_move_resize(p);
+	      return 0;
+	    }
 	}
     }
 
@@ -190,8 +202,14 @@ main_client_configure(Client *c)
 	 - wm_get_offsets_size(c->wm, EAST,  NULL, False)
 	 - wm_get_offsets_size(c->wm, WEST,  NULL, False);
 
+#ifdef USE_ALT_INPUT_WIN
+       c->height = c->wm->dpy_height - c->y - h - offset_south - main_client_manage_toolbars_for_fullscreen(c, False);
+#else
        c->height = c->wm->dpy_height - c->y - h - offset_south;
        main_client_manage_toolbars_for_fullscreen(c, False);
+#endif
+
+
      }
 
    dbg("%s() configured as %i*%i+%i+%i, frame size is %i\n", 
@@ -286,7 +304,7 @@ main_client_reparent(Client *c)
   XClearWindow(w->dpy, c->frame);
 
   XSetWindowBorderWidth(w->dpy, c->window, 0);
-  XAddToSaveSet(w->dpy, c->window);
+  XAddToSaveSet(w->dpy, c->window); 
   XSelectInput(w->dpy, c->window, ColormapChangeMask|PropertyChangeMask);
   XReparentWindow(w->dpy, c->window, c->frame, offset_west, offset_north);
 }
@@ -378,6 +396,9 @@ main_client_toggle_fullscreen(Client *c)
 
   wm_activate_client(c); /* Reactivate, stacking order slightly different  */
 
+  /* start pinging for this change of state */
+  if (c->wm->config->ping_aggressive)
+    ewmh_ping_client_start (c);
 }
 
 
@@ -570,20 +591,33 @@ void main_client_button_press(Client *c, XButtonEvent *e)
       return;
    }
 
-   stack_enumerate(w, p)
+   if (w->config->super_modal)
      {
-       if (p->trans == c && (p->flags & CLIENT_IS_MODAL_FLAG))
+       stack_enumerate(w, p)
 	 {
-	   MBList *button_item = client_get_button_list_item_from_event(c, e);
-
-	   /* In the precense of a modal transient dialog ignore 
-	    * certain buttons. 
-	    *
-	    * For now its just the custom type. May make sense to
-            * Add more.
-	   */
-	   if (button_item && button_item->id == BUTTON_ACTION_CUSTOM)
-	     return;
+	   if (p->trans == c && (p->flags & CLIENT_IS_MODAL_FLAG))
+	     {
+	       MBList *button_item = client_get_button_list_item_from_event(c, e);
+	       /* In the precense of a modal transient dialog ignore 
+		* certain buttons. 
+		*
+		*/
+	       if ((button_item && button_item->id == BUTTON_ACTION_CUSTOM)
+		   || (button_item && button_item->id == BUTTON_ACTION_MIN)
+		   || (button_item && button_item->id == BUTTON_ACTION_CLOSE))
+		 {
+		   if (button_item->id == BUTTON_ACTION_CLOSE)
+		     {
+		       /* initiate pinging the app anyway for close button */
+		       if (c->has_ping_protocol && c->pings_pending == -1) 
+			 {
+			   c->pings_pending = 0;
+			   w->n_active_ping_clients++;
+			 }
+		     }
+		   return;
+		 }
+	     }
 	 }
      }
 
@@ -721,6 +755,8 @@ main_client_iconize(Client *c)
   Wm *w = c->wm;
   Client *p = NULL;
 
+  dbg("%s() called on %s\n", __func__, c->name);
+
   client_set_state(c, IconicState);
   c->flags |= CLIENT_IS_MINIMIZED;
 
@@ -728,6 +764,8 @@ main_client_iconize(Client *c)
   stack_enumerate(w, p)
     if (p->trans == c)
       p->iconize(p);
+
+  main_client_unmap(c);
 }
 
 void
@@ -735,8 +773,13 @@ main_client_show(Client *c)
 {
   Wm     *w = c->wm;
 
-   dbg("%s() called on %s\n", __func__, c->name);
+  Client *visible_app_client = NULL;
+
+  dbg("%s() called on %s\n", __func__, c->name);
    
+  if (c->flags & CLIENT_NO_FOCUS_ON_MAP)
+    visible_app_client = wm_get_visible_main_client(w);
+
    if (w->flags & DESKTOP_RAISED_FLAG) 
      {
        c->flags |= CLIENT_NEW_FOR_DESKTOP;
@@ -757,18 +800,45 @@ main_client_show(Client *c)
       care of painels etc as it can use active client as a 'watermark' 
    */
    stack_move_top(c);
-   
+
    stack_dump(w);
 
    if (!c->mapped)
      {
        if (c->flags & CLIENT_IS_MINIMIZED)
 	 {
+	   Client *p = NULL;
+
+	   /* Reset state from Iconized */
 	   client_set_state(c, NormalState);
 	   c->flags &= ~CLIENT_IS_MINIMIZED;
+
+	   /* Make sure any dialogs are shown too */
+	   stack_enumerate(w, p)
+	     if (p->trans == c)
+	       {
+		 p->show(p);
+		 /* To handle delay mapping.. */
+		 XMapSubwindows(w->dpy, p->frame);
+		 XMapWindow(w->dpy, p->frame);
+	       }
+	 }
+
+       if (c->flags & CLIENT_NO_FOCUS_ON_MAP)
+	 {
+	   if (visible_app_client)
+	     {
+	       stack_move_above_client (c, visible_app_client->below);
+	       c->flags |= CLIENT_DELAY_MAPPING;
+	     }
+	   c->flags &= ~CLIENT_NO_FOCUS_ON_MAP;
+	 }
+
+       if (!(c->flags & CLIENT_DELAY_MAPPING))
+	 {
+	   XMapSubwindows(w->dpy, c->frame);
+	   XMapWindow(w->dpy, c->frame);
 	 }
-       XMapSubwindows(w->dpy, c->frame);
-       XMapWindow(w->dpy, c->frame);
      }
 
    c->mapped = True;
