@@ -78,6 +78,10 @@ static struct frame_lookup_t
   { "dialog-east",   FRAME_DIALOG_EAST  },
   { "dialog-west",   FRAME_DIALOG_WEST  },
   { "dialog-south",  FRAME_DIALOG_SOUTH },
+  { "dialog-nt-north",  FRAME_DIALOG_NT_NORTH },
+  { "dialog-nt-east",   FRAME_DIALOG_NT_EAST  },
+  { "dialog-nt-west",   FRAME_DIALOG_NT_WEST  },
+  { "dialog-nt-south",  FRAME_DIALOG_NT_SOUTH },
   { "utility-min",   FRAME_UTILITY_MIN  },
   { "utility-max",   FRAME_UTILITY_MAX  },
   { "menu",          FRAME_MENU         },
@@ -111,7 +115,7 @@ lookup_frame_type(char *name)
       return frame_lookup[i].id;
   return -1;
 }
-
+ 
 void theme_frame_icon_paint(MBTheme *t, Client *c, 
 			    MBPixbufImage *img_dest, 
 			    int x, int y)
@@ -131,15 +135,16 @@ void theme_frame_icon_paint(MBTheme *t, Client *c,
     {
       if (c->icon)
 	{
-	  Window win_crap;
-	  int icon_w, icon_h, crap;
-	  XGetGeometry(t->wm->dpy, c->icon, &win_crap, &crap, &crap, 
-		       &icon_w, &icon_h, &crap, &crap);
+	  Window       win_foo;
+	  int          foo;
+	  unsigned int icon_w, icon_h, ufoo;
+	  XGetGeometry(t->wm->dpy, c->icon, &win_foo, &foo, &foo, 
+		       &icon_w, &icon_h, &ufoo, &ufoo);
 	  img = mb_pixbuf_img_new_from_drawable(t->wm->pb, 
 						c->icon, 
 						c->icon_mask, 
 						0, 0, 
-						icon_w, icon_h);
+						(int)icon_w, (int)icon_h);
 	} 
     }
 
@@ -149,7 +154,7 @@ void theme_frame_icon_paint(MBTheme *t, Client *c,
   if (img->width != 16 || img->height != 16) 
     {
       MBPixbufImage *tmp_img;
-      tmp_img = mb_pixbuf_img_scale_down(t->wm->pb, img, 16, 16); 
+      tmp_img = mb_pixbuf_img_scale(t->wm->pb, img, 16, 16); 
       mb_pixbuf_img_free(t->wm->pb, img);
       img = tmp_img;
     }
@@ -296,11 +301,12 @@ theme_frame_button_paint(MBTheme *theme,
 		{
 		  img_backing = mb_pixbuf_img_rgb_new(pb, button_w, button_h);
 
-		  mb_pixbuf_img_copy(pb, img_backing,
-				     theme->img_caches[frame_type],
-				     button_x, button_y,
-				     button_w, button_h,
-				     0, 0 );
+		  if (!theme->disable_pixbuf_cache)
+		    mb_pixbuf_img_copy(pb, img_backing,
+				       theme->img_caches[frame_type],
+				       button_x, button_y,
+				       button_w, button_h,
+				       0, 0 );
 
 		}
 
@@ -383,6 +389,17 @@ theme_has_message_decor( MBTheme *theme )
     return True;
 }
 
+Bool
+theme_has_borders_only_decor( MBTheme *theme )
+{
+  MBThemeFrame *frame = (MBThemeFrame *)list_find_by_id(theme->frames, 
+							FRAME_DIALOG_NT_NORTH);
+  if (frame == NULL) 
+    return False;
+  else
+    return True;
+}
+
 static void
 _theme_paint_gradient(MBTheme*       theme, 
 		      MBThemeLayer*  layer_cur, 
@@ -599,16 +616,19 @@ theme_frame_paint( MBTheme *theme,
     {
     case FRAME_MAIN_SOUTH:
     case FRAME_DIALOG_SOUTH:
+    case FRAME_DIALOG_NT_SOUTH:
     case FRAME_MSG_SOUTH:
       decor_idx = SOUTH;
       break;
     case FRAME_MAIN_EAST: 
     case FRAME_DIALOG_EAST:
+    case FRAME_DIALOG_NT_EAST:
     case FRAME_MSG_EAST:
       decor_idx = EAST;
       break;
     case FRAME_MAIN_WEST: 
     case FRAME_DIALOG_WEST:
+    case FRAME_DIALOG_NT_WEST:
     case FRAME_MSG_WEST:
       decor_idx = WEST;
       break;
@@ -746,7 +766,8 @@ theme_frame_paint( MBTheme *theme,
   
   if (c->backing_masks[MSK_NORTH] != None &&
       ( frame_type == FRAME_MAIN || frame_type == FRAME_DIALOG 
-	|| frame_type == FRAME_MSG || frame_type == FRAME_DIALOG_NORTH)
+	|| frame_type == FRAME_MSG || frame_type == FRAME_DIALOG_NORTH
+	|| frame_type == FRAME_DIALOG_NT_NORTH)
       )
     mb_pixbuf_img_render_to_mask(theme->wm->pb, img, 
 				 c->backing_masks[MSK_NORTH],
@@ -754,7 +775,8 @@ theme_frame_paint( MBTheme *theme,
   
   if (c->backing_masks[MSK_SOUTH] != None  &&
       ( frame_type == FRAME_MAIN_SOUTH || frame_type == FRAME_DIALOG_SOUTH 
-	|| frame_type == FRAME_MSG_SOUTH )
+	|| frame_type == FRAME_MSG_SOUTH 
+	|| frame_type == FRAME_DIALOG_NT_SOUTH)
       )
     mb_pixbuf_img_render_to_mask(theme->wm->pb, img, 
 				 c->backing_masks[MSK_SOUTH],
@@ -762,7 +784,7 @@ theme_frame_paint( MBTheme *theme,
   
   if (c->backing_masks[MSK_EAST] != None  &&
       ( frame_type == FRAME_MAIN_EAST || frame_type == FRAME_DIALOG_EAST 
-	|| frame_type == FRAME_MSG_EAST )
+	|| frame_type == FRAME_MSG_EAST || frame_type == FRAME_DIALOG_NT_EAST)
       )
     mb_pixbuf_img_render_to_mask(theme->wm->pb, img, 
 				 c->backing_masks[MSK_EAST],
@@ -770,7 +792,7 @@ theme_frame_paint( MBTheme *theme,
   
   if (c->backing_masks[MSK_WEST] != None  &&
       ( frame_type == FRAME_MAIN_WEST || frame_type == FRAME_DIALOG_WEST 
-	|| frame_type == FRAME_MSG_WEST )
+	|| frame_type == FRAME_MSG_WEST || frame_type == FRAME_DIALOG_NT_WEST )
       )
     mb_pixbuf_img_render_to_mask(theme->wm->pb, img, 
 				 c->backing_masks[MSK_WEST],
@@ -783,7 +805,7 @@ theme_frame_paint( MBTheme *theme,
   
   /* No point caching frame images which dont have buttons */
   
-  if (decor_idx != NORTH)
+  if (decor_idx != NORTH || theme->disable_pixbuf_cache == True)
     theme_img_cache_clear (theme, frame_type);
   
   /* Now paint text onto pixmap */
@@ -814,6 +836,10 @@ theme_frame_paint( MBTheme *theme,
 			     (c->name_is_utf8) ? MB_ENCODING_UTF8 : MB_ENCODING_LATIN,
 			     text_render_opts);
       dbg("%s() rendered text\n", __func__);
+
+      /* FIXME: we really need an mb_font_unset_color() here..? */
+      mb_col_unref(layer_label->label->col);
+      layer_label->label->font->col = NULL;
     }
 
   /* Finally put the drawable on the decoration frame background */
@@ -864,7 +890,8 @@ theme_frame_menu_get_dimentions(MBTheme* theme, int *w, int *h)
   stack_enumerate(theme->wm, p)
   {
     if ((p->type == MBCLIENT_TYPE_APP || p->type == MBCLIENT_TYPE_DESKTOP) 
-	&& p->name && p->mapped
+	&& p->name 
+	/* && p->mapped */  /* Include iconsized clients in sizing */
 	&& p != wm_get_visible_main_client(theme->wm))
       {
 	int this_width = mb_font_get_txt_width (frame->font,
@@ -927,6 +954,8 @@ _theme_frame_menu_paint_text_entry(MBTheme      *theme,
 			 (entry->name_is_utf8) ? 
 			    MB_ENCODING_UTF8 : MB_ENCODING_LATIN,
 			 MB_FONT_RENDER_OPTS_CLIP_TRAIL);
+
+  mb_col_unref(color); 	/* set_color refs */
 }
 
 void
@@ -1091,7 +1120,7 @@ theme_frame_menu_paint(MBTheme* theme, Client *c)
       p = (Client*)item->data;
       if (p->type == MBCLIENT_TYPE_APP 
 	  && p->name && !(p->flags & CLIENT_IS_DESKTOP_FLAG)
-	  && p->mapped
+	  && p->mapped 
 	  && p != wm_get_visible_main_client(w))
 	{
 	  theme_frame_icon_paint(theme, p, img, 
@@ -2197,6 +2226,7 @@ mbtheme_new (Wm *w)
   
   t->have_toolbar_panel = False;
   
+
   return t;
 }
 
@@ -2272,27 +2302,10 @@ void
 mbtheme_switch (Wm   *w, 
 		char *new_theme_name)
 {
-  Client *c, *p, *orig;
-  Window win_active = None;
-  Bool reset_new_for_desktop_flag = False;
+  Client *p = NULL;
 
   XGrabServer(w->dpy);
 
-  orig = c = p = w->stack_top_app; 
-
-  if (w->stack_top_app)
-    {
-      win_active = w->stack_top_app->window;
-      if (w->stack_top_app->flags & CLIENT_NEW_FOR_DESKTOP)
-	reset_new_for_desktop_flag = True;
-    }
-
-  if (w->flags & DESKTOP_RAISED_FLAG
-      && (orig = wm_get_desktop(w)))
-    {
-      win_active = orig->window;
-    }
-
   /* now the fun part */
   mbtheme_free(w, w->mbtheme);
 
@@ -2316,6 +2329,8 @@ mbtheme_switch (Wm   *w,
 	}
     }
 
+  /* Now resize ( due to new frames ) + repaint everything */
+
   stack_enumerate(w, p)
     {
       client_buttons_delete_all(p);
@@ -2328,8 +2343,6 @@ mbtheme_switch (Wm   *w,
 	  /*  Old theme may have been shaped and this one not. 
 	   *  Therefore we 'clear' any possible shapes set on 
 	   *  the window frames.  
-	   *  Im not sure if this is the best way to do this but
-	   *  SHAPE docs serverely lacking :(
 	   */
 	  
 	  xregion = XCreateRegion ();
@@ -2355,32 +2368,13 @@ mbtheme_switch (Wm   *w,
       p->move_resize(p);
       p->redraw(p, False);
     }
-    
 
-  /* fix if desktop is shown */
-  if (win_active)
-    {
-      Client *found = wm_find_client(w, win_active, WINDOW);
-      if (found)
-	{
-	  if (found->type == MBCLIENT_TYPE_DESKTOP)
-	    {
-	      w->flags &= ~DESKTOP_RAISED_FLAG; /* Clear desktop flag */
-	      wm_toggle_desktop(w);
-	    } 
-	  else 
-	    {
-	      wm_activate_client(found);
-	      if (reset_new_for_desktop_flag)
-		found->flags |= CLIENT_NEW_FOR_DESKTOP;
-	    }
-	}
-    }
+  ewmh_update_rects(w); /* theme *could* affect this */
+    
+  XSync(w->dpy, False);
 
   XUngrabServer(w->dpy);
 
-  XSync(w->dpy, False);
-
   comp_engine_render(w, None);
 }
 
@@ -2489,10 +2483,10 @@ mbtheme_init (Wm   *w,
   chdir(theme_path);
 
   XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME], XA_STRING, 8,
-		  PropModeReplace, theme_path, strlen(theme_path));
+		  PropModeReplace, (unsigned char*)theme_path, strlen(theme_path));
 
   XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME_NAME], XA_STRING, 8,
-		  PropModeReplace, theme_name, strlen(theme_name));
+		  PropModeReplace, (unsigned char*)theme_name, strlen(theme_name));
 
   free(theme_path);
 
@@ -2526,6 +2520,13 @@ mbtheme_init (Wm   *w,
      }       
 
    w->mbtheme = mbtheme_new(w);
+
+   if (get_attr(root_node, "cache") 
+       && !strcasecmp(get_attr(root_node, "cache"), "false"))
+     {
+       if (!getenv("MB_THEME_ALWAYS_CACHE"))
+	 w->mbtheme->disable_pixbuf_cache = True;
+     }
    
    for(n = root_node->kids; n != NULL; n = n->next)
    {
