@@ -68,6 +68,10 @@ wm_new(int argc, char **argv)
    w->n_active_ping_clients    = 0;
    w->next_click_is_not_double = True;
 
+#ifdef USE_SM
+   sm_connect(w); 	/* previous_cliend_id */
+#endif
+
    sattr.event_mask =  SubstructureRedirectMask
                        |SubstructureNotifyMask
                        |StructureNotifyMask
@@ -172,7 +176,7 @@ wm_new(int argc, char **argv)
 void
 wm_usage(char *progname)
 {
-   printf("usage: %s %s [options ...]\n", progname, VERSION);
+   printf("usage: %s [options ...]\n", progname);
    printf("\t-display          <string> \n");
    printf("\t-theme            <string> \n");
    printf("\t-use_titlebar     <yes|no>\n");
@@ -182,15 +186,17 @@ wm_usage(char *progname)
 #endif
    printf("\t-use_dialog_mode  <free|static|const-horiz>\n");
    printf("\t-use_desktop_mode <decorated|plain>\n");
+   printf("\t-use_super_modal  <yes|no>\n");
    printf("\t-force_dialogs    <comma seperated list of window titles>\n");
-   /*
-   printf("\t-ping_handler     <string>\n");
-   */
+#ifdef USE_SM
+   printf("\t--sm-client-id    <session id>\n");
+#endif
+
 #ifdef STANDALONE
    printf("\t-titlebar_panel   <x11 geometry>\n");
 #endif
    printf("\n");
-   printf("Compile time options:\n");
+   printf("Compile time features;\n");
 #ifdef DEBUG
    printf("\tdebug build                      yes\n");
 #else
@@ -269,8 +275,16 @@ wm_usage(char *progname)
    printf("\tXRM support                      no\n");
 #endif
 
-   printf("\nVisit http://projects.o-hand.com/matchbox for more info.\n");
-   printf("(c) 2004 OpenedHand Ltd\n");
+#ifdef USE_SM
+   printf("\tSession support                  yes\n");
+#else
+   printf("\tSession support                  no\n");
+#endif   
+
+   printf("\nmatchbox-window-manager %s \nCopyright (C) 2005 Matthew Allum, OpenedHand Ltd.\n", VERSION);
+   printf("This is free software; see the source for copying conditions.\n");
+   printf("Visit http://projects.o-hand.com/matchbox for more info.\n");
+
    exit(0);
 }
 
@@ -295,7 +309,8 @@ wm_load_config (Wm   *w,
    w->config->no_cursor        = False;
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
-   w->config->ping_handler     = NULL;
+   w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
+   w->config->ping_aggressive = getenv("MB_AGGRESSIVE_PING") ? True : False;
 
 #ifdef USE_COMPOSITE
    w->config->dialog_shade = True;
@@ -349,6 +364,13 @@ wm_load_config (Wm   *w,
 	  continue;
 	}
 #endif
+      if (!strcmp ("-use_super_modal", argv[i])) 
+	{
+	  if (++i>=*argc) wm_usage (argv[0]);
+	  if (strcmp(argv[i], "yes") == 0)
+            w->config->super_modal = True;   
+	  continue;
+	}
       if (!strcmp("-use_dialog_mode", argv[i])) 
 	{
 	  if (++i>=*argc) wm_usage (argv[0]);
@@ -401,6 +423,16 @@ wm_load_config (Wm   *w,
 	  w->config->force_dialogs = argv[i];
 	  continue;
 	}
+
+#ifdef USE_SM
+   if (!strcmp ("--sm-client-id", argv[i]))
+     {
+       if (++i>=*argc) wm_usage (argv[0]);
+       w->config->sm_client_id = strdup(argv[i]);
+       continue;
+     }
+#endif
+
       wm_usage (argv[0]);
    }
 
@@ -423,7 +455,7 @@ wm_load_config (Wm   *w,
    char              *type;
    XrmValue          value;
    
-   static int opTableEntries = 9;
+   static int opTableEntries = 11;
    static XrmOptionDescRec opTable[] = {
       {"-theme",       ".theme",           XrmoptionSepArg, (XPointer) NULL},
       {"-use_titlebar",".titlebar",        XrmoptionSepArg, (XPointer) NULL},
@@ -434,6 +466,8 @@ wm_load_config (Wm   *w,
       {"-use_desktop_mode",".desktop",     XrmoptionSepArg, (XPointer) NULL},
       {"-titlebar_panel",  ".titlebarpanel", XrmoptionSepArg, (XPointer) NULL},
       {"-force_dialogs",  ".forcedialogs", XrmoptionSepArg, (XPointer) NULL},
+      {"--sm-client-id",  ".session",      XrmoptionSepArg, (XPointer) NULL},
+      {"-use_super_modal", ".supermodal",  XrmoptionSepArg, (XPointer) NULL},
    };
 
    XrmInitialize();
@@ -455,7 +489,8 @@ wm_load_config (Wm   *w,
    w->config->no_cursor        = False;
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
-   w->config->ping_handler     = NULL;
+   w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
+   w->config->ping_aggressive  = getenv("MB_AGGRESSIVE_PING") ? True : False;
 
    if (XrmGetResource(rDB, "matchbox.display",
 		      "Matchbox.Display",
@@ -527,6 +562,17 @@ wm_load_config (Wm   *w,
 	 }
      }   
 
+   if (XrmGetResource(rDB, "matchbox.supermodal", "Matchbox.Supermodal",
+		      &type, &value) == True)
+   {
+      if(strncmp(value.addr, "yes", (int) value.size) == 0)
+      {
+	 dbg("%s() TURNING SUPER MODAL ON\n", __func__);
+	 w->config->super_modal = True;
+      }
+   }
+
+
    /* 
     *  Composite matchbox always uses lowlighting 
     */
@@ -602,6 +648,19 @@ wm_load_config (Wm   *w,
      free(geom);
    }
 #endif
+
+#ifdef USE_SM
+   if (XrmGetResource(rDB, "matchbox.session",
+		      "Matchbox.session",
+		      &type, &value) == True)
+     {
+       w->config->sm_client_id = (char *)malloc(sizeof(char)*(value.size+1));
+       strncpy(w->config->sm_client_id, value.addr, (int) value.size);
+       w->config->sm_client_id[value.size] = '\0';
+       dbg("%s() got sm_client_id: %s ", __func__, w->config->sm_client_id);
+     }
+#endif   
+
 }
 #endif /* NO_XRM */
 
@@ -614,6 +673,11 @@ wm_init_existing(Wm *w)
    XWindowAttributes attr;
    Client *c;
    
+   /* set blank hints */
+   ewmh_update_rects(w); 
+   ewmh_update_lists(w); 
+   ewmh_update_desktop_hint(w);
+
    XQueryTree(w->dpy, w->root, &dummyw1, &dummyw2, &wins, &nwins);
    for (i = 0; i < nwins; i++) {
       XGetWindowAttributes(w->dpy, wins[i], &attr);
@@ -657,34 +721,55 @@ wm_find_client(Wm *w, Window win, int mode)
 
 /* Grab an X Event - block but With a timeout */
 static Bool
-get_xevent_timed(Display        *dpy, 
+get_xevent_timed(Wm             *w,
 		 XEvent         *event_return, 
 		 struct timeval *tv)
 {
   if (tv->tv_usec == 0 && tv->tv_sec == 0)
     {
-      XNextEvent(dpy, event_return);
+#ifndef USE_SM
+      XNextEvent(w->dpy, event_return);
       return True;
+#else
+      tv->tv_sec = 5;
+#endif
     }
 
-  XFlush(dpy);
+  XFlush(w->dpy);
 
-  if (XPending(dpy) == 0) 
+  if (XPending(w->dpy) == 0) 
     {
-      int fd = ConnectionNumber(dpy);
+      int fd = ConnectionNumber(w->dpy);
       fd_set readset;
       FD_ZERO(&readset);
       FD_SET(fd, &readset);
 
+#ifdef USE_SM
+      if (w->sm_ice_fd != -1)
+	{
+	  FD_SET(w->sm_ice_fd, &readset);
+	  if (w->sm_ice_fd > fd) 
+	    fd = w->sm_ice_fd; 	/* for +1 below */
+	}
+#endif
       if (select(fd+1, &readset, NULL, NULL, tv) == 0) 
-	return False;
-      else {
-	XNextEvent(dpy, event_return);
-	return True;
+	{
+	  return False;
+	}
+      else 
+	{
+#ifdef USE_SM
+	  if ( w->sm_ice_fd != -1 && FD_ISSET(w->sm_ice_fd, &readset) )
+	    {
+	      sm_process_event(w);
+	      return False;
+	    }
+#endif
+	  XNextEvent(w->dpy, event_return);
+	  return True;
       }
-
     } else {
-      XNextEvent(dpy, event_return);
+      XNextEvent(w->dpy, event_return);
       return True;
     }
 }
@@ -785,7 +870,7 @@ wm_event_loop(Wm* w)
 	tvt.tv_sec = 1;
 #endif
 
-      if (get_xevent_timed(w->dpy, &ev, &tvt))
+      if (get_xevent_timed(w, &ev, &tvt))
 	{
 
 	  switch (ev.type) 
@@ -861,8 +946,8 @@ wm_event_loop(Wm* w)
 
 
 #ifndef NO_PING
-	/* check for hung apps every two seconds - they dont last long.. */
-	if (++hung_app_timer > 2 && w->n_active_ping_clients)
+	/* check for hung apps every PING_CHECK_FREQ seconds */
+	if (++hung_app_timer > PING_CHECK_FREQ && w->n_active_ping_clients)
 	  {
 	    hung_app_timer = 0;
 	    ewmh_hung_app_check(w);
@@ -1220,6 +1305,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
      {
        /* Dont know any thing about this window. Let it have what it 
         * wants and leave. 
+	* Can be a window initally mapping, thus the trap as it could
+        * of already dissapeared.
         */
        dbg("%s() could find source client %ix%i\n", __func__,
 	   e->width, e->height);
@@ -1229,7 +1316,9 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
        xwc.height  = req_h;
        xwc.sibling = e->above;
        xwc.stack_mode = e->detail;
+       misc_trap_xerrors();
        XConfigureWindow(w->dpy, e->window, e->value_mask, &xwc);
+       misc_untrap_xerrors();
        return;
      }
 
@@ -1544,6 +1633,20 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
 	   w->flags ^= DEBUG_COMPOSITE_VISIBLE_FLAG;
 #endif
 	   break;
+#ifndef NO_KBD
+	 case MB_CMB_KEYS_RELOAD:
+	   keys_reinit(w); 
+	   break;
+#endif
+
+#ifdef USE_COMPOSITE
+	 case MB_CMD_COMPOSITE:
+	   if (w->comp_engine_disabled)
+	     comp_engine_reinit(w);
+	   else
+	     comp_engine_deinit(w);
+	   break;
+#endif
 	 }
        return;
      }
@@ -1614,7 +1717,25 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
       if (!misc_untrap_xerrors() && success)
 	if ((new_trans_client = wm_find_client(w, trans_win, WINDOW)) != NULL)
 	  {
+	    Client *p = new_trans_client;
+
+	    /* Dont let clients be change transiency to themselves
+             * either directly or recursively.
+	    */
+	    while (p != NULL)
+	      {
+		if (p == c)
+		  {
+		    dbg("%s() CLIENT WARNING: %s ( %li ) transient for self\n",
+			__func__, c->name, c->window);
+		    c->trans = NULL;
+		    return; 
+		  }
+		p = p->trans;
+	      }
+
 	    c->trans = new_trans_client;
+
 	    return;
 	  }
 
@@ -1631,7 +1752,7 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 
       misc_trap_xerrors(); 	/* avoid possible X Errors */
 
-      c->name = ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
+      c->name = (char*)ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
 
       if (!misc_untrap_xerrors())
 	{
@@ -1648,11 +1769,35 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 	  update_titlebar = True;
 	}
     }
+  else  if (e->atom == w->atoms[WM_PROTOCOLS])
+    {
+      int orig_flags = c->flags;
+
+      dbg("%s() WM_PROTOCOLS changed, flags %i\n", __func__, orig_flags);
+
+      client_get_wm_protocols(c);
+
+      dbg("%s() WM_PROTOCOLS changed, flags now %i\n", __func__, c->flags);
+
+      /* If flags have changed, likely means a WM_CONTEXT_HELP|etc has changed 
+       * and thus titlebar needs a repaint.
+      */
+      if (c->flags != orig_flags)
+	{
+	  dbg("%s() WM_PROTOCOLS changed flags, repainting\n", __func__);
+
+	  /* Buttons will get recreated on demand via repaint */
+	  client_buttons_delete_all(c);
+	  update_titlebar = True;
+	}
+    }
   else  if (e->atom == w->atoms[WM_CHANGE_STATE])
     {
       dbg("%s() state change, name is %s\n", __func__, c->name);
       if(client_get_state(c) == WithdrawnState)
-	c->destroy(c);
+	{
+	  wm_remove_client(w, c);
+	}
     }
   else if (e->atom == w->atoms[CM_TRANSLUCENCY])
     {
@@ -1690,7 +1835,7 @@ wm_make_new_client(Wm *w, Window win)
    unsigned long n, extra;
    int           format, status;
    Client       *c = NULL, *t = NULL;
-   XWMHints     *wmhints;
+   XWMHints     *wmhints = NULL;
    int           mwm_flags = 0;
 
    XGrabServer(w->dpy);
@@ -1710,10 +1855,16 @@ wm_make_new_client(Wm *w, Window win)
      {
        /* Use EWMH Window Type Hint to figure out window type */
 
+       misc_trap_xerrors();
+
        status = XGetWindowProperty(w->dpy, win, w->atoms[WINDOW_TYPE], 
 				   0L, 1000000L, 0, XA_ATOM, 
 				   &realType, &format,
 				   &n, &extra, (unsigned char **) &value);
+
+       if (misc_untrap_xerrors()) /* An X error occured - win deleted ? */
+	 goto end;
+       
        if (status == Success)
 	 {
 	   if (realType == XA_ATOM && format == 32)
@@ -1745,11 +1896,19 @@ wm_make_new_client(Wm *w, Window win)
 		   if (c == NULL) goto end;
 		   c->flags ^= CLIENT_TITLE_HIDDEN_FLAG;
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG]
+			|| value[0] == w->atoms[WINDOW_TYPE_MENU])
 		 {
 		   dbg("%s() got type dialog atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
+
+		   /* check to see if we're a 'menu' dialog  */
+		   if (value[0] == w->atoms[WINDOW_TYPE_MENU])
+		     {
+		       dbg("%s() Got menu hint\n", __func__);
+		       c->flags |= CLIENT_IS_MENU_DIALOG;
+		     }
 		 }
 	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
 		 {
@@ -1806,6 +1965,7 @@ wm_make_new_client(Wm *w, Window win)
 		     break; 
 		   }
 	    }
+	    if (wmhints) XFree(wmhints);
 	 }
       }
 
@@ -1879,7 +2039,7 @@ wm_make_new_client(Wm *w, Window win)
 
    /* Let window know were all done */
 
-   ewmh_state_set(c);
+   ewmh_state_set(c); 		/* XXX This is likely not needed */
 
    client_set_state(c, NormalState);
 
@@ -1900,20 +2060,32 @@ wm_remove_client(Wm *w, Client *c)
 
   XGrabServer(w->dpy);
 
-  misc_trap_xerrors(); 	/* below likely to genrate errors */
+  misc_trap_xerrors(); 	/* below very likely to genrate X errors */
 
   /* Set win state to withdrawn and reparent to root. 
    * If below is not done - suttle bugs appear with toolkits ( GTK )
    */
   client_set_state(c, WithdrawnState);
-  XReparentWindow(w->dpy, c->window, w->root, 0, 0); 
+
+  /* Below is likely not needed ( window will have already unmapped )
+   * and likely to fire an X error, but we do it anyway to be extra safe.
+   * 
+   * In gtk quickly hiding/showing window repeatadly it can seem gtk does't
+   * always call the unmap and a wierd ghost window is left.
+  */
+  XUnmapWindow(w->dpy, c->window);
+
+  /* Reparent now unmanaged win to root */
+  XReparentWindow(w->dpy, c->window, w->root, c->x, c->y); 
   XRemoveFromSaveSet(w->dpy, c->window);
 
+  /* sync here so any (likely) lingering X errors are trapped */
+  XSync(w->dpy, False);
+
   misc_untrap_xerrors();
 
   c->destroy(c);
 
-  XSync(w->dpy, False);
   XUngrabServer(w->dpy);
 }
 
@@ -2051,10 +2223,20 @@ wm_update_layout(Wm         *w,
 		 p->height += change_amount;
 		 p->y      -= change_amount;
 		 p->move_resize(p);
-		 theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
-		 theme_pixmap_cache_clear_all(w->mbtheme);
+		 /* XXX shouldn't be any need to redraw here as 
+                  * width won't have changed so no need to repaint 
+                  * entire toolbar. 
+                  *
+                  * It *could* break on if the side decoration themes
+                  * are somehow a function of height. No themes I know
+                  * of do this and we avoid quite a lot of cpu avoiding
+                  * this.
+                  *
+		  * theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
+		  * theme_pixmap_cache_clear_all(w->mbtheme);
+		  * main_client_redraw(p, False);
+		 */
 		 client_deliver_config(p);
-		 main_client_redraw(p, False); /* force title redraw */
 		 break;
 	       case MBCLIENT_TYPE_PANEL :
 		 if (p->flags & CLIENT_DOCK_NORTH
@@ -2072,6 +2254,8 @@ wm_update_layout(Wm         *w,
        }
      else
        {
+	 /* toolbar changes here */
+
 	 dbg("%s(): restack NORMAL comparing %i <= %i for %s\n",
 	     __func__, p->y, client_changed->y, p->name);
 	 if ( (p->y <= client_changed->y) 
@@ -2084,10 +2268,14 @@ wm_update_layout(Wm         *w,
 	       case MBCLIENT_TYPE_APP :
 		 p->height += change_amount;
 		 p->move_resize(p);
-		 theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
-		 theme_pixmap_cache_clear_all(w->mbtheme);
+		 /*
+		  * See above as to why this is commented out.
+                  *
+                  * theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
+		  * theme_pixmap_cache_clear_all(w->mbtheme);
+		  * main_client_redraw(p, False);
+                  */
 		 client_deliver_config(p);
-		 main_client_redraw(p, False); /* force title redraw */
 		 break;
 	       case MBCLIENT_TYPE_TOOLBAR :
 		 p->y += change_amount;
@@ -2142,10 +2330,11 @@ wm_update_layout(Wm         *w,
        }
    }
 
+ ewmh_update_rects(w);
+
  XSync(w->dpy, False);
  XUngrabServer(w->dpy);
 
- ewmh_update_rects(w);
 }
 
 
@@ -2188,6 +2377,10 @@ wm_activate_client(Client *c)
 {
   Wm     *w;
   Client *client_to_focus = c;
+  Bool    set_desktop_show_hint = False;
+#ifndef NO_PING
+  Client *prev_app_client = NULL;
+#endif
 
   if (c == NULL) return; /* its possible for this to happen :( */
 
@@ -2195,6 +2388,13 @@ wm_activate_client(Client *c)
 
   dbg("%s() called for %s\n", __func__, c->name);
 
+#ifndef NO_PING
+  if (w->config->ping_aggressive)
+    {
+      prev_app_client = wm_get_visible_main_client(w);
+    }
+#endif
+
   XGrabServer(w->dpy);
 
   c->show(c); /* Set 'relative' pos in stack, map windows
@@ -2204,16 +2404,28 @@ wm_activate_client(Client *c)
       __func__, (w->flags & DESKTOP_RAISED_FLAG));
 
 
+
   if (c->type == MBCLIENT_TYPE_APP || c->type == MBCLIENT_TYPE_DESKTOP) 
     {
       /* As matchbox works around 'main' windows ( apps/main and desktop wins).
 	 We need to sync extra stuff up when displaying a new one.
        */
 
+      Bool switching_from_to_fullscreen = False;
+
       /* save focus state for transient dialogs of prev showing main win */
 
       if (w->stack_top_app && w->stack_top_app != c)
-	w->stack_top_app->hide(w->stack_top_app); 
+	{
+	  if ( (w->stack_top_app->flags & CLIENT_FULLSCREEN_FLAG
+		&& !(c->flags & CLIENT_FULLSCREEN_FLAG))
+	       || 
+	       ( !(w->stack_top_app->flags & CLIENT_FULLSCREEN_FLAG)
+		&& c->flags & CLIENT_FULLSCREEN_FLAG ) )
+	    switching_from_to_fullscreen = True;
+
+	  w->stack_top_app->hide(w->stack_top_app); 
+	}
 
       /* If this client has a saved dialog focus state, load it */
 
@@ -2253,6 +2465,9 @@ wm_activate_client(Client *c)
 	  dbg("%s() clearing desktop flag\n", __func__);
 	  w->flags &= ~DESKTOP_RAISED_FLAG;
 	  w->stack_top_app = c;      
+
+	  set_desktop_show_hint = True;
+
 	}
       else
 	{
@@ -2267,12 +2482,22 @@ wm_activate_client(Client *c)
 	    {
 	      stack_move_above_client(w->have_titlebar_panel, NULL);
 	    }
+
+	  set_desktop_show_hint = True;
+
 	}
 
-      /* We only set active for main clients - this could be wrong, 
-         what about transient for root 'apps' - maybe it should just
-         follow focused client ? */
-      ewmh_set_active(w);
+      /* Set active main client, set focus handles active win hint */
+      ewmh_set_current_app_window(w);
+
+#ifdef USE_ALT_INPUT_WIN
+      /* If switching from/to fullscreen, then there could be an 
+       * input window transient for a transient for root dialog and
+       * it therefore needs to be resized ( the IM ).
+      */
+      if (switching_from_to_fullscreen &&  c->type == MBCLIENT_TYPE_APP)
+	  main_client_manage_toolbars_for_fullscreen(c, False);
+#endif
 
     }
   else if (c->type == MBCLIENT_TYPE_DIALOG)
@@ -2309,6 +2534,23 @@ wm_activate_client(Client *c)
 	  */
 	  wm_stack_dialogs_for_transient(w, wm_get_visible_main_client(w));
 	}
+
+      /* Check if this is transient for an app/desktop thats
+       * not currently at the top of the stack. In which case
+       * dont try and focus it.
+      */
+      if (c->trans != NULL) 	/* not trans for root */
+	{
+	  Client *dialog_parent = c->trans;
+
+	  while (dialog_parent->trans != NULL)
+	    dialog_parent = dialog_parent->trans;
+
+	  if ( (dialog_parent->type == MBCLIENT_TYPE_APP
+		|| dialog_parent->type == MBCLIENT_TYPE_DESKTOP)
+	       && wm_get_visible_main_client(w) != dialog_parent)
+	    client_to_focus = NULL;
+	}
     }
 
 
@@ -2327,9 +2569,10 @@ wm_activate_client(Client *c)
       stack_move_above_client(c, NULL);
     }
 
-  ewmh_update(w);
+  ewmh_update_lists(w);
 
-  client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
+  if (set_desktop_show_hint) 
+    ewmh_update_desktop_hint(w);
 
   stack_dump(w);
 
@@ -2337,8 +2580,40 @@ wm_activate_client(Client *c)
 
   stack_sync_to_display(w);
 
+  if (c->flags & CLIENT_DELAY_MAPPING) /* See dialog show() source */
+    {
+      c->flags &= ~CLIENT_DELAY_MAPPING;
+      XMapSubwindows(w->dpy, c->frame);
+      XMapWindow(w->dpy, c->frame);
+    }
+
+
+  client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
+
   comp_engine_client_show(w, c);
 
+#ifndef NO_PING
+  if (w->config->ping_aggressive)
+    {
+      Client *new_app_client;
+
+      new_app_client = wm_get_visible_main_client(w);
+
+      if (prev_app_client != new_app_client)
+	{
+	  if (new_app_client && new_app_client->type != MBCLIENT_TYPE_DESKTOP)
+	    {
+	      ewmh_ping_client_start (new_app_client);
+	    }
+
+	  if (prev_app_client)
+	    {
+	      ewmh_ping_client_stop (prev_app_client);
+	    }
+	}
+    }
+#endif
+
   XSync(w->dpy, False);	    
 
   XUngrabServer(w->dpy);
@@ -2445,6 +2720,8 @@ wm_toggle_desktop(Wm *w)
        dbg("%s() showing desktop\n", __func__);
        wm_activate_client(wm_get_desktop(w));
      }
+
+
 }
 
 void
